
/***********************************************************************
* SCCS id: @(#)IFF_to_CDF.h	1.28 17:40:38 04/27/00 
************************************************************************/

/* include files */

#include <stddef.h> 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <setjmp.h> 
#include <time.h> 
#include <math.h>
#include <ctype.h>
 

#include "/cluster/cdf/include/cdf.h"

/* environment variables */

#define GEN_BY  "Generated by UK CDHF"
#define SW_VER  "QMW CDF-writing software, QIFF,  V1.28 [ 04/27/00 ]"
#define IVL_TOLERANCE  100
#define NO_DATA  "_NoData"
#define NO_DATA_TXT  "No data detected by QMW cdf writing sw"
#define NO_MODE  "_NotProvided"
#define NO_MODE_TXT  "No instrument mode detected by QMW cdf writing sw"


/* sizes */

#define SIZE_0  0
#define TEXT_LEN 120
#define NAME_LEN 40
#define WORD_LEN 305
#define FULL_NAME_LEN 240
#define NUMBER_OF_WORDS 1000
#define MAX_NUMBER_OF_VARS 300
#define MAX_NUMBER_OF_DIMS 10
#define HALF_MIN 30000

/* constants */

#define BLOCK '\xfd'
#define IVL_FILL -1.e31
#define GAP 1
#define DATA 0
#define NOT_FOUND 1
#define FOUND 0
#define PAIR 2
#define DELIM '\"'
#define PLACEHOLDER "TO BE OVERWRITTEN"
#define END_OF_DATA 500
#define QMW_OK 0
#define QMW_ERROR -1
#define QMW_WARNING 1
#define NO_DELETE -1


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* set up globals */

FILE * logfile;
char log_fname[FULL_NAME_LEN];
long CDF_delete=0;
jmp_buf jmp_env;
int dbg = 0;   /* flag to turn on extra prints */

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* set up structures */

struct QiSFName {
          char fname[NAME_LEN];          /* file name root */
          char sub_strings[NAME_LEN];    /* dummy to hold sc, rr, iii */
          char *sc;                      /* s/c id, e.g. Cn or CL */
          char *rr;                      /* resolution, e.g. SP or PP */
          char *iii;                     /* instrument abbreviation */
          long year;                     /* year */
          long month;                    /* month */
          long day;                      /* day */ 
          char *extn;                    /* optional extension to file name */                  
};

/*------------------------------------------------------*/

struct QiSParams {
       long n_recs;                           /* number of records in cdf */
       long n_vars;                           /* number of variables in cdf */
       long EPOCH_var_num;                    /* varnum for Epoch */
       long recvary[MAX_NUMBER_OF_VARS];      /* VARY/NOVARY for all vars */
       long n_list;                           /* number of VARY vars */
       long n_fills;                          /* number of fill values */
       long n_data;                           /* number values per input line */
       long n_bad;                            /* number of bad data lines */
       long list_vars[MAX_NUMBER_OF_VARS];    /* varnums for all VARY vars */
       long var_dim[MAX_NUMBER_OF_VARS];      /* dimension of all vars */
       long dim_size[MAX_NUMBER_OF_VARS][MAX_NUMBER_OF_DIMS]; 
                                              /* sizes of dims for each var */
       long n_cells[MAX_NUMBER_OF_VARS];      /* (dim * size) for each var */
       long data_type[MAX_NUMBER_OF_VARS];    /* cdf data type of each var */
       size_t data_len[MAX_NUMBER_OF_VARS];   /* byte length of each variable */
       long n_chars[MAX_NUMBER_OF_VARS];      /* number of elements for chars */
       double msec_day;                       /* msec at start of day */
       double msec_last_rec;                  /* msec of last record */
       char fill_cdf[MAX_NUMBER_OF_VARS][WORD_LEN];
                                              /* fill values to place in CDF */
       char fill_iff[MAX_NUMBER_OF_VARS][WORD_LEN];
                                              /* fill values found in iff */
          
};

/*------------------------------------------------------*/

struct QiSBuffer {
          size_t buff_len;        /* number of bytes in buffer */
          unsigned char *buff;    /* pointer to buffer for complete record */
                                  /* tmp for i/o of different data types */
          short sint_data;
          short sint_array[NUMBER_OF_WORDS];
          long int_data;        
          long int_array[NUMBER_OF_WORDS];
          unsigned int uint_data;        
          unsigned int uint_array[NUMBER_OF_WORDS];
          double double_data;
          double double_array[NUMBER_OF_WORDS];
          float real_data;
          float real_array[NUMBER_OF_WORDS];
          char char_data[WORD_LEN];
          unsigned char uchar_data[WORD_LEN];
};

/*------------------------------------------------------*/

struct QiSIO {
          int n_words;                      /* number of words found in line */
          char *line ;                /* input line */
          char *line_cpy ;   /* tmp space for input line copy*/
          char words[NUMBER_OF_WORDS][WORD_LEN]; /* array of words from line */

};


/*------------------------------------------------------*/

struct QiSInterval {
          double ivl_ave;                 /* average size of ivl */
          long   ivl_num;                 /* number of ivls used for ave */
          double ivl_max;                 /* largest  ivl */
          double ivl_min;                 /* smallest ivl */
          double ivl_start;               /* start time of ivl */
          double *ivl_array;              /* value for each record */
};

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/*  FUNCTION  PROTOTYPES  */

/**********************************/

long QiIFFtoCDF (char *iff_fname,
                 char *cdf_fname,
                 char *log_fname,
                 struct QiSFName *QiSname);

/**********************************/
                 
void QiErrorHandler (long status, 
                   char *note);

/**********************************/

double QiGetDay0(long year, 
                 long month, 
                 long day);

/**********************************/
long QiFNameParser(struct QiSFName *name);

/**********************************/
long QiComposeName(char *varname, 
                 char *stem,
                 struct QiSFName *name);
                 
/**********************************/

long QiLineParser(struct QiSIO *io);

/**********************************/

void QiStatusHandler (CDFstatus status, 
                    char *note);

/**********************************/

long QiWriteHalfInterval(struct QiSFName *name, 
                         struct QiSInterval *Hi,
                         long num_records,
                         double data_res);

/**********************************/

long QiVersionStamp();

/**********************************/

long QiSetupCDF(struct QiSFName *name,
               struct QiSParams *cdf,
               struct QiSBuffer *rec);
               
/**********************************/

long QiInitRecords(struct QiSParams *cdf, 
                    struct QiSInterval *QiShi);
                 
/**********************************/

long QiIFFScan(FILE *f, 
             struct QiSFName *name, 
             struct QiSParams *cdf, 
             struct QiSIO *io,
             double  *data_res);

/**********************************/

long QiIFFRead(FILE *f, 
             struct QiSParams *cdf, 
             struct QiSBuffer *rec, 
             struct QiSIO *io,
             struct QiSInterval *Hi,
             double   data_res);
             
/**********************************/

char * QiCtoISO(char *time,
                char *iso_time );

/**********************************/

char * QiFillLine(struct QiSIO *io,
                  FILE *stream);

/**********************************/

long QiPutGattr(struct QiSIO *io,
             struct QiSBuffer *rec);

/**********************************/

CDFstatus QiGetVarInfo (struct QiSParams *QiScdf, 
                        struct QiSBuffer *QiSrec,
                        char *varname,
                        long varnum);
                        
/**********************************/

long QiWriteEpochAttrs(struct QiSFName *QiSname, 
                       struct QiSParams *QiScdf);

/**********************************/

long QiInterval(struct QiSInterval *QiShi, 
                long *place, 
                long *post_gap,
                long recnum,
                double msec,
                double gap_tol);
                
/**********************************/

long QiStrcpy(char *outstring, 
              char *instring);
             
/**********************************/

long QiCleanUp(struct QiSFName *QiSname, 
               struct QiSParams *QiScdf);

/**********************************/

unsigned int QiStrToUInt(char *instring);

/**********************************/

int QiStrToInt(char *instring); 

/**********************************/

char * QiStrFree( char * str);

/**********************************/
